@@ -1,4 +1,4 @@
-# graphics devices in base R, plus those in Cairo, cairoDevice, tikzDevice
+# graphics devices in base R, plus those in Cairo, tikzDevice
 auto_exts = c(
   bmp = 'bmp', postscript = 'eps', pdf = 'pdf', png = 'png', svg = 'svg',
   jpeg = 'jpeg', pictex = 'tex', tiff = 'tiff', win.metafile = 'wmf',
@@ -11,16 +11,17 @@ auto_exts = c(
   CairoJPEG = 'jpeg', CairoPNG = 'png', CairoPS = 'eps', CairoPDF = 'pdf',
   CairoSVG = 'svg', CairoTIFF = 'tiff',
 
-  Cairo_pdf = 'pdf', Cairo_png = 'png', Cairo_ps = 'eps', Cairo_svg = 'svg',
-
-  svglite = 'svg',
+  svglite = 'svg', gridSVG = 'svg',
 
   ragg_png = 'png',
 
   tikz = 'tex'
 )
 
-dev2ext = function(x) {
+# choose a file extension according to the device, unless fig.ext is provided
+dev2ext = function(options) {
+  if (length(ext <- options$fig.ext)) return(ext)
+  x = options$dev
   res = auto_exts[x]
   if (any(idx <- is.na(res))) {
     for (i in x[idx]) dev_get(i)
@@ -60,6 +61,10 @@ dev_get = function(dev, options = opts_current$get(), dpi = options$dpi[1]) {
     pdf = grDevices::pdf,
     png = function(...) png(..., res = dpi, units = 'in'),
     svg = grDevices::svg,
+    gridSVG = function(filename, width, height, ...) {
+      # use svg() only for redrawing the plot, and will use gridSVG::grid.export() later
+      grDevices::svg(filename, width, height)
+    },
     pictex = grDevices::pictex,
     tiff = function(...) tiff(..., res = dpi, units = 'in'),
     win.metafile = grDevices::win.metafile,
@@ -81,11 +86,6 @@ dev_get = function(dev, options = opts_current$get(), dpi = options$dpi[1]) {
     CairoPDF = load_device('CairoPDF', 'Cairo'),
     CairoSVG = load_device('CairoSVG', 'Cairo'),
 
-    Cairo_pdf = load_device('Cairo_pdf', 'cairoDevice'),
-    Cairo_png = load_device('Cairo_png', 'cairoDevice'),
-    Cairo_ps = load_device('Cairo_ps', 'cairoDevice'),
-    Cairo_svg = load_device('Cairo_svg', 'cairoDevice'),
-
     svglite = load_device('svglite', 'svglite'),
 
     # similar to load_device(), but the `dpi` argument is named `res`
@@ -113,10 +113,10 @@ quartz_dev = function(type, dpi) {
 }
 
 # a wrapper of the tikzDevice::tikz device
-tikz_dev = function(...) {
+tikz_dev = function(..., engine = getOption('tikzDefaultEngine')) {
   loadNamespace('tikzDevice')
   packages = switch(
-    getOption('tikzDefaultEngine'),
+    engine,
     pdftex = getOption('tikzLatexPackages'),
     xetex = getOption('tikzXelatexPackages'),
     luatex = getOption('tikzLualatexPackages')
@@ -156,7 +156,12 @@ plot2dev = function(plot, name, dev, device, path, width, height, options) {
   do.call(device, c(list(path, width = width, height = height), dargs))
   showtext(options)  # maybe begin showtext and set options
   print(plot)
+  # hack: if the device is gridSVG, save the plot to a temp path (with suffix ~)
+  path2 = if (dev == 'gridSVG') paste0(path, '~')
+  if (!is.null(path2)) do.call(gridSVG::grid.export, c(list(name = path2), dargs))
   dev.off()
+  # move the temp svg file to `path`
+  if (!is.null(path2)) file.rename(path2, path)
 
   # Cairo::CairoPS always adds the extension .ps, even if you have specified an
   # extension like .eps (https://github.com/yihui/knitr/issues/1364)
@@ -168,7 +173,9 @@ plot2dev = function(plot, name, dev, device, path, width, height, options) {
   # compile tikz to pdf
   if (dev == 'tikz') {
     patch_tikz_tex(path)
-    if (options$external) path = tinytex::latexmk(path, getOption('tikzDefaultEngine'))
+    if (options$external) path = tinytex::latexmk(
+      path, dargs$engine %n% getOption('tikzDefaultEngine')
+    )
   }
 
   fig_process(options$fig.process, path, options)
@@ -203,7 +210,7 @@ get_dargs = function(dargs, dev) {
   dargs
 }
 
-# this is mainly for Cairo and cairoDevice
+# this is mainly for Cairo
 load_device = function(name, package, dpi = NULL) {
   dev = getFromNamespace(name, package)
   # dpi is for bitmap devices; units must be inches!
@@ -423,7 +430,9 @@ par2 = function(x) {
 #' Markdown syntax, to embed an external image. Chunk options related to
 #' graphics output that work for normal R plots also work for these images, such
 #' as \code{out.width} and \code{out.height}.
-#' @param path A character vector of image paths.
+#' @param path A character vector of image paths. Both local file paths and web
+#'   paths are supported. Note that the \code{auto_pdf} and \code{dpi} arguments
+#'   are not supported for web paths.
 #' @param auto_pdf Whether to use PDF images automatically when the output
 #'   format is LaTeX. If \code{TRUE}, then e.g. \file{foo/bar.png} will be
 #'   replaced by \file{foo/bar.pdf} if the latter exists. This can be useful
@@ -433,11 +442,16 @@ par2 = function(x) {
 #'   inches) of the images. This will be their actual width in pixels, divided
 #'   by \code{dpi}. If not provided, the chunk option \code{dpi} is used; if
 #'   \code{NA}, the output width will not be calculated.
+#' @param rel_path Whether to automatically convert absolute paths to relative
+#'   paths. If you know for sure that absolute paths work, you may set this
+#'   argument or the global option \code{knitr.graphics.rel_path} to
+#'   \code{FALSE}.
 #' @param error Whether to signal an error if any files specified in the
 #'   \code{path} argument do not exist and are not web resources.
 #' @note This function is supposed to be used in R code chunks or inline R code
-#'   expressions. You are recommended to use forward slashes (\verb{/}) as path
-#'   separators instead of backslashes in the image paths.
+#'   expressions. For local images, you are recommended to use relative paths
+#'   with forward slashes instead of backslashes (e.g., \file{images/fig1.png}
+#'   instead of \file{/Users/me/code/images/fig1.png}).
 #'
 #'   The automatic calculation of the output width requires the \pkg{png}
 #'   package (for PNG images) or the \pkg{jpeg} package (for JPEG images). The
@@ -449,9 +463,18 @@ par2 = function(x) {
 #' @export
 include_graphics = function(
   path, auto_pdf = getOption('knitr.graphics.auto_pdf', FALSE), dpi = NULL,
+  rel_path = getOption('knitr.graphics.rel_path', TRUE),
   error = getOption('knitr.graphics.error', TRUE)
 ) {
   path = native_encode(path)  # https://d.cosx.org/d/420524
+  if (any(i <- xfun::is_abs_path(path)) && rel_path && !is.null(d <- opts_knit$get('output.dir'))) {
+    path[i] = xfun::relative_path(path[i], d, error = FALSE)
+    if (any(j <- xfun::is_abs_path(path[i]))) warning(
+      'It is highly recommended to use relative paths for images. ',
+      'You had absolute paths: ', quote_vec(path[i][j])
+    )
+  }
+  path = path.expand(path) # https://github.com/rstudio/rmarkdown/issues/1053
   if (auto_pdf && is_latex_output()) {
     path2 = with_ext(path, 'pdf')
     i = file.exists(path2)
@@ -460,7 +483,7 @@ include_graphics = function(
   # relative paths can be tricky in child documents, so don't error (#1957)
   if (child_mode()) error = FALSE
   if (error && length(p <- path[!xfun::is_web_path(path) & !file.exists(path)])) stop(
-    'Cannot find the file(s): ', paste0('"', p, '"', collapse = '; ')
+    'Cannot find the file(s): ', quote_vec(p)
   )
   structure(path, class = c('knit_image_paths', 'knit_asis'), dpi = dpi)
 }
