@@ -25,10 +25,9 @@ rst2pdf = function(input, command = 'rst2pdf', options = '') {
 #' Knits the input file and compiles to an output format using Pandoc.
 #' @inheritParams knit
 #' @param to Character string giving the Pandoc output format to use.
-#' @param pandoc_wrapper An R function used to call Pandoc; by default, if
-#'   \pkg{rmarkdown} is installed, this uses
-#'   \code{rmarkdown::\link[rmarkdown]{pandoc_convert}()}, otherwise it
-#'   \code{\link{pandoc}()}.
+#' @param pandoc_wrapper An R function used to call Pandoc. If \code{NULL} (the
+#'   default), \code{rmarkdown::\link[rmarkdown]{pandoc_convert}()} will be used
+#'   if \pkg{rmarkdown} is installed, otherwise \code{\link{pandoc}()}.
 #' @param ... Options to be passed to the \code{pandoc_wrapper} function.
 #' @param encoding Ignored (always assumes UTF-8).
 #' @author Trevor L. Davis
@@ -50,12 +49,12 @@ knit2pandoc = function(
 #' Knit the input Rnw or Rrst document, and compile to PDF using
 #' \code{tinytex::\link[tinytex]{latexmk}()} or \code{\link{rst2pdf}()}.
 #' @inheritParams knit
-#' @param compiler A character string giving the LaTeX engine used to
-#'   compile the tex document to PDF. For an Rrst file, setting \code{compiler} to
+#' @param compiler A character string giving the LaTeX engine used to compile
+#'   the tex document to PDF. For an Rrst file, setting \code{compiler} to
 #'   \code{'rst2pdf'} will use \code{\link{rst2pdf}} to compile the rst file to
 #'   PDF using the ReportLab open-source library.
-#' @param ... Options to be passed to \code{tinytex::\link[tinytex]{latexmk}} or
-#'   \code{\link{rst2pdf}}.
+#' @param ... Options to be passed to \code{tinytex::\link[tinytex]{latexmk}()}
+#'   or \code{\link{rst2pdf}()}.
 #' @author Ramnath Vaidyanathan, Alex Zvoleff and Yihui Xie
 #' @return The filename of the PDF file.
 #' @note The \code{output} argument specifies the output filename to be passed
@@ -84,17 +83,62 @@ knit2pdf = function(
   with_ext(out, 'pdf')
 }
 
-#' Convert markdown to HTML using knit() and markdownToHTML()
+#' Convert an \file{Rnw} document to PDF
+#'
+#' Call \code{\link{knit}()} to compile the \file{.Rnw} input to \file{.tex},
+#' and then \code{tinytex::\link[tinytex]{latexmk}()} to convert \file{.tex} to
+#' \file{.pdf}.
+#'
+#' This function is similar to \code{\link{knit2pdf}()}, with the following differences:
+#' \enumerate{
+#'   \item The default compiler is "xelatex" instead of "pdflatex".
+#'   \item \code{output} uses the file extension ".pdf" instead of ".tex".
+#'   \item Before knitting, it tries to remove the \code{output} file and will throw a clear error if the file cannot be removed.
+#'   \item \code{output} could be under any dir, not necessarily the same directory as \code{input}.
+#'   \item It cleans up intermediate files by default, including the ".tex" file.
+#'   \item It stops knitting when any error occurs (by setting the chunk option \code{error = FALSE}).
+#' }
+#' @inheritParams knit
+#' @param output Path of the PDF output file. By default, it uses the same name
+#'   as the \code{input}, but changes the file extension to ".pdf".
+#' @param compiler,... The LaTeX engine and other arguments to be passed to
+#'   \code{tinytex::\link[tinytex]{latexmk}()}. The default compiler is
+#'   \code{xelatex}.
+#' @param clean If \code{TRUE}, the intermediate files will be removed.
+#' @param error If \code{FALSE}, knitting stops when any error occurs.
+#' @return The \code{output} file path.
+#' @export
+rnw2pdf = function(
+  input, output = with_ext(input, 'pdf'), compiler = 'xelatex',
+  envir = parent.frame(), quiet = FALSE, clean = TRUE, error = FALSE, ...
+) {
+  # On Windows, when tweaking the content, users may forget to close the PDF
+  # file (thus can't be written). Since knitting may take quite some time, it's
+  # better to check the write permission of the output file in advance.
+  if (xfun::file_exists(output) && !file.remove(output)) stop(
+    "The file '", output, "' cannot be removed (may be locked by a PDF reader)."
+  )
+  old = opts_chunk$set(error = error)
+  on.exit(opts_chunk$set(old), add = TRUE)
+  file_tex = knit(input, envir = envir, quiet = quiet)
+  if (clean) on.exit(file.remove(file_tex), add = TRUE)
+  file_pdf = tinytex::latexmk(file_tex, engine = compiler, clean = clean, ...)
+  if (!xfun::same_path(output, file_pdf)) file.rename(file_pdf, output)
+  output
+}
+
+#' Convert markdown to HTML using knit() and mark_html()
 #'
 #' This is a convenience function to knit the input markdown source and call
-#' \code{\link[markdown]{markdownToHTML}()} in the \pkg{markdown} package to
-#' convert the result to HTML.
+#' \code{markdown::\link[markdown]{mark_html}()} in the \pkg{markdown}
+#' package to convert the result to HTML.
 #' @inheritParams knit
-#' @param ... Options passed to \code{\link[markdown]{markdownToHTML}}.
-#' @param force_v1 Boolean; whether to force rendering the input document as an R
-#'   Markdown v1 document, even if it is for v2.
+#' @param ... Options passed to
+#'   \code{markdown::\link[markdown]{mark_html}()}.
+#' @param force_v1 Boolean; whether to force rendering the input document as an
+#'   R Markdown v1 document, even if it is for v2.
 #' @export
-#' @seealso \code{\link{knit}}, \code{\link[markdown]{markdownToHTML}}
+#' @seealso \code{\link{knit}}, \code{markdown::\link[markdown]{mark_html}}
 #' @return If the argument \code{text} is NULL, a character string (HTML code)
 #'   is returned; otherwise the result is written into a file and the filename
 #'   is returned.
@@ -108,11 +152,15 @@ knit2pdf = function(
 #' if (interactive()) browseURL('test.html')
 #'
 #' unlink(c('test.Rmd', 'test.html', 'test.md'))
-knit2html = function(input, output = NULL, ..., envir = parent.frame(), text = NULL,
-                     quiet = FALSE, encoding = 'UTF-8', force_v1 = FALSE) {
+knit2html = function(
+  input, output = NULL, ..., envir = parent.frame(), text = NULL,
+  quiet = FALSE, encoding = 'UTF-8', force_v1 = getOption('knitr.knit2html.force_v1', FALSE)
+) {
+  if (is_cran_check() && !has_package('markdown'))
+    return(vweave_empty(input, .reason = 'markdown'))
+
   if (!force_v1 && is.null(text)) {
-    signal = if (is_R_CMD_check()) warning2 else stop2
-    if (length(grep('^---\\s*$', head(read_utf8(input), 1)))) signal(
+    if (length(grep('^---\\s*$', head(read_utf8(input), 1)))) warning2(
       'It seems you should call rmarkdown::render() instead of knitr::knit2html() ',
       'because ', input, ' appears to be an R Markdown v2 document.'
     )
@@ -120,17 +168,25 @@ knit2html = function(input, output = NULL, ..., envir = parent.frame(), text = N
   out = knit(input, text = text, envir = envir, quiet = quiet)
   if (is.null(text)) {
     output = with_ext(if (is.null(output) || is.na(output)) out else output, 'html')
-    markdown::markdownToHTML(out, output, encoding = 'UTF-8', ...)
+    mark_html(out, output, ...)
     invisible(output)
-  } else markdown::markdownToHTML(text = out, ...)
+  } else mark_html(text = out, ...)
 }
 
-knit2html_v1 = function(...) knit2html(..., force_v1 = TRUE)
+mark_html = function(...) {
+  if (packageVersion('markdown') < '1.3') stop(
+    "The 'markdown' package version >= 1.3 is required."
+  )
+  markdown::mark_html(...)
+}
 
 #' Knit an R Markdown document and post it to WordPress
 #'
 #' This function is a wrapper around the \pkg{RWordPress} package. It compiles
-#' an R Markdown document to HTML and post the results to WordPress.
+#' an R Markdown document to HTML and post the results to WordPress. Please note
+#' that \pkg{RWordPress} has not been updated for several years, which is
+#' \href{https://github.com/yihui/knitr/issues/1866}{not a good sign}. For
+#' blogging with R, you may want to try the \pkg{blogdown} package instead.
 #' @param input Filename of the Rmd document.
 #' @param title Title of the post.
 #' @param ... Other meta information of the post, e.g. \code{categories = c('R',
@@ -140,10 +196,10 @@ knit2html_v1 = function(...) knit2html(..., force_v1 = TRUE)
 #'   for syntax highlighting of source code and output. The first element
 #'   applies to source code, and the second applies to text output. By default,
 #'   both are \code{FALSE}.
-#' @param action Whether to create a new post, update an existing post, or create a new
-#'   page.
-#' @param postid If \code{action} is \code{editPost}, the post id \code{postid} must be
-#'   specified.
+#' @param action Whether to create a new post, update an existing post, or
+#'   create a new page.
+#' @param postid If \code{action} is \code{editPost}, the post id \code{postid}
+#'   must be specified.
 #' @param publish Boolean: publish the post immediately?
 #' @inheritParams knit
 #' @export
@@ -159,9 +215,17 @@ knit2wp = function(
   input, title = 'A post from knitr', ..., envir = parent.frame(), shortcode = FALSE,
   action = c('newPost', 'editPost', 'newPage'), postid, publish = TRUE
 ) {
+  do.call('library', list(package = 'RWordPress', character.only = TRUE))
+  xfun::do_once(
+    warning2(
+      'This function is based on the RWordPress package, which is no longer actively ',
+      'maintained (https://github.com/yihui/knitr/issues/1866). For blogging with R, ',
+      'you may try the blogdown package instead.'
+    ), 'knitr.knit2wp.warning'
+  )
   out = knit(input, envir = envir); on.exit(unlink(out))
   content = file_string(out)
-  content = markdown::markdownToHTML(text = content, fragment.only = TRUE)
+  content = mark_html(text = content)
   shortcode = rep(shortcode, length.out = 2L)
   if (shortcode[1]) content = gsub(
     '<pre><code class="([[:alpha:]]+)">(.+?)</code></pre>',
@@ -189,7 +253,6 @@ knit2wp = function(
   # if we are editing the post, also include the argument for postid
   if (action == "editPost") WPargs = c(postid = postid, WPargs)
 
-  do.call('library', list(package = 'RWordPress', character.only = TRUE))
   do.call(action, args = WPargs)
 }
 
