@@ -15,7 +15,7 @@ render_markdown()
 
 img_output = function(path, opts = list()) {
   opts = opts_chunk$merge(opts)
-  wrap(knit_print(include_graphics(path, error = FALSE)), opts)
+  sew(knit_print(include_graphics(path, error = FALSE)), opts)
 }
 
 assert('include_graphics() includes custom images correctly', {
@@ -29,6 +29,11 @@ assert('include_graphics() includes custom images correctly', {
 hook_src = knit_hooks$get("source")
 options_ = list(engine = "r", prompt = FALSE, highlight = TRUE)
 
+assert('Lengh of fences are satisfied', {
+  (hook_src("", options_) %==% "\n\n```r\n\n```\n\n")
+  (hook_src("```", options_) %==% "\n\n````r\n```\n````\n\n")
+})
+
 assert('Attributes for source can be specified class.source and attr.source', {
   (hook_src("1", c(options_, class.source = "a b")) %==% "\n\n```{.r .a .b}\n1\n```\n\n")
   (hook_src("1", c(options_, attr.source = ".a .b")) %==% "\n\n```{.r .a .b}\n1\n```\n\n")
@@ -60,6 +65,15 @@ assert('Attributes for souce can be specified class.source and attr.source', {
     "\n\n```{.a b='1'}\n1\n```\n\n")
 })
 
+hook_chunk = knit_hooks$get("chunk")
+
+assert('Chunks are enclosed by fenced divs when needed.', {
+  (hook_chunk('', list(class.chunk=NULL)) %==% '')
+  (hook_chunk('', list(class.chunk="")) %==% '::: \n\n:::')
+  (hook_chunk('', list(class.chunk="foo")) %==% '::: {.foo}\n\n:::')
+  (hook_chunk(':::', list(class.chunk="foo")) %==% ':::: {.foo}\n:::\n::::')
+})
+
 knit_hooks$restore()
 
 
@@ -67,8 +81,14 @@ x = "1.png"
 w = h = 1
 ex = "style='margin: 0;'"
 cap = "foo"
-opt <- function(w = NULL, h =NULL, ex = NULL, cap = NULL, show = 'asis', ...) {
-  list(out.width = w, out.height = h, out.extra = ex, fig.cap = cap, fig.show = show, ...)
+opt = function(
+  w = NULL, h = NULL, ex = NULL, cap = NULL, show = 'asis',
+  fig.align = 'default', ...
+) {
+  list(
+    out.width = w, out.height = h, out.extra = ex,
+    fig.cap = cap, fig.show = show, fig.align = fig.align, ...
+  )
 }
 
 assert("Include a plot by pandoc md", {
@@ -80,3 +100,11 @@ assert("Include a plot by pandoc md", {
   (hook_plot_md_pandoc(x, opt(w = w, cap = cap, ex = ex)) %==%
     sprintf("![%s](1.png){width=%s %s}", cap, w, ex))
 })
+
+assert("fig.alt does not break office document", {
+  old = opts_knit$get('rmarkdown.pandoc.to')
+  opts_knit$set(rmarkdown.pandoc.to = "docx")
+  (suppressWarnings(hook_plot_md(x, opt())) %==% "![](1.png)")
+  (suppressWarnings(hook_plot_md(x, opt(fig.alt = "bar"))) %==% "![](1.png)")
+  opts_knit$set('rmarkdown.pandoc.to' = old)
+})
